(** * Tutoriel 1 - Prouver avec Coq : Logique propositionnelle *)



(** ** I. Un premier exemple - Commutativité de la conjonction.*)

(** Souvenez-vous, c'est le premir exemple que vous avez vu en agda*)

(**   En coq, une preuve est une succession de tactiques, chaque tactique réduisant le
    premier sous-but à une nouvelle liste (éventuellement vide) de sous-buts.*)
(** Ces tactiques correspondent à peu près aux raccourcis clavier d'agda, comme on peut voir sur cet exemple*)

Lemma conj_comm : forall P Q, P /\ Q -> Q /\ P.
Proof.
  intros P Q H. (*Ctrl-C Ctrl-R*)
  destruct H as [fst snd]. (*Ctrl-C Ctrl-C sur H*)
  split. (*Ctrl-C Ctrl-R*)
   + exact snd. (*Ctrl-C Ctrl-space après avoir rempli snd dans le trou*)
   + exact fst.
Qed.

(* Contrairement à Agda, on ne voit pas le lambda-term qu'on est en train de construire.
Si on veut le voir, on peut print. La syntaxe n'est pas là même, mais moralement c'est le même lambda-term*)
Print conj_comm.



(** Cette preuve montre l'utilisation de plusieurs tactiques que nous
    allons expliquer plus précisément au fur et à mesure des feuilles
    d'exercices.

    Notez déjà l'utilisation massive des bullets [+] afin d'identifier
    les points de branchement dans une preuve.
    Les autres bullets possibles sont [*], [-], [**], [+++], etc.
    Si une bullet est utilisée pour un sous-but, cette même bullet doit être
    utilisée pour tous les autres sous-buts du même niveau.
    Il est également possible d'utiliser les accolades pour délimiter un but. *)

(** Dans cette feuille d'exercices, nous nous intéressons uniquement à des 
    énoncés exprimables dans la logique propositionnelle.
    Vous n'avez donc besoin que des tactiques suivantes :
     - intro / intros : introduction de l'implication ->
     - apply H : pour utiliser un énoncé, une hypothèse
     - exact H / assumption : la règle nommée "axiome"
     - left et right : pour l'introduction de la disjonction \/
     - split : pour l'introduction de la conjonction /\
     - destruct H : pour l'élimination de la conjonction /\, la disjonction \/,
                    de [False]
     - unfold d in H : pour dérouler une définition [d] dans une hypothèse [H].
*)

(** Dans Coq, le type des propositions est [Prop].
    Nous déclarons trois objets arbitraires de ce type. *)

Variable P : Prop.
Variable Q : Prop.
Variable R : Prop.

(** La requête [Check] peut être utilisée pour vérifier que les expressions ont
    un sens, mais aussi de voir leur type.
    Les requêtes suivantes montrent les notations Coq pour la disjonction (\/),
    la conjonction (/\) et l'implication (->). *)

Check True.
Check P \/ Q.
Check False /\ P.
Check P -> Q -> R.



(** ** II. Commutativité de la disjonction. *)

Goal P \/ Q -> Q \/ P.
Proof.
  intro H.
  destruct H.
   + right. exact H.
   + left.  exact H.
Qed.

(** ** III. Distributivité *)

(** *** Exercice 1 - A vous de jouer ! *)

Goal P /\ (Q \/ R) -> (P /\ Q) \/ (P /\ R).
Proof.
Admitted.

Goal (P /\ Q) \/ (P /\ R) -> P /\ (Q \/ R).
Proof.
Admitted.

(** ** IV. Une forme de commutativité pour l'implication.

    A noter ici :
      - l'associativité de l'implication
      - la variante [intros] de [intro]. *)

Goal (P -> Q -> R) -> (Q -> P -> R).
Proof.
  intros H HQ HP.
  apply H.
   + exact HP.
   + exact HQ.
Qed.

(** ** V. Hypothèses multiples

    Prouvez l'équivalence suivante, en gardant à l'esprit que
    [P <-> Q] est une notation pour [(P -> Q) /\ (Q -> P)].
    Vous pouvez le vérifier en utilisant la tactique [unfold iff]. *)

(** *** Exercice 2 - A vous de jouer ! *)

Goal (P -> Q -> R) <-> (P /\ Q -> R).
Proof.
Admitted.

(** Note : En Coq, on écrit généralement [P -> Q -> R] plutôt que [P /\ Q -> R],
    puisque la première forme est plus facile à travailler en utilisant les
    tactiques fournies. *)

(** ** VI. Négation

    La négation [~ P] est une notation pour [P -> False]. *)

Goal P -> ~ ~ P.
Proof.
  intros H Hn.
  unfold not in Hn.
  apply Hn.
  exact H.
Qed.

(** *** Exercice 3 - A vous de jouer ! *)

Goal (~P /\ ~Q) -> ~(P \/ Q).
Proof.
Admitted.

Goal ~(P \/ Q) -> (~P /\ ~Q).
Proof.
Admitted.

Goal (~P \/ ~Q) -> ~(P /\ Q).
Proof.
Admitted.

(** *** Exercice 4 - Avec le modus ponens *)

(** Note : Les quantifications [forall P : Prop] sont
           des quantifications d'ordre supérieur.
           Nous quittons donc la logique propositionnelle. *)

Lemma modus_ponens : forall P Q : Prop, P -> (P -> Q) -> Q.
Proof.
Admitted.

(** Démontrer à nouveau l'énoncé suivant, mais cette fois en utilisant
    [modus_ponens]. *)
Lemma impl_not : forall P : Prop, P -> ~ ~ P.
Proof.
Admitted.

(** Démontrer l'énoncé suivant à partir de [impl_not]. *)
Lemma P_notP_contradiction : forall P : Prop, ~ (P /\ ~ P).
Proof.
Admitted.

Lemma not_and : forall P Q : Prop, P \/ Q -> ~(~ P /\ ~Q).
Proof.
Admitted.

(** ** VII.  Raisonnement classique

    Comme Agda, Coq implémente une logique constructive.
    Cela veut dire que la logique interne de Coq n'a pas l'axiome du tiers exclu,
    ou un équivalent. Il n'est également pas possible de prouver le tiers exclu. *)

(** *** Exercice 5 - Tiers exclu avec Coq *)

(** Pour le raisonnement classique, il faut donc déclarer un axiome.
    Note : [RAA] signifie pour Reductio Ad Absurdum.

    Ces objectifs de preuve sont un peu plus difficiles. *)

Axiom RAA : forall A:Prop, ~~A -> A.

Goal ~(P /\ Q) -> (~P \/ ~Q).
Proof.
Admitted.

(** [LEM] pour Law of Excluded Middle. *)
Lemma LEM : P \/ ~P.
Proof.
Admitted.

Goal (P -> Q) \/ (Q -> P).
Proof.
Admitted.

(** Merci à David Baelde et Catherine Dubois. *)
