(**** Complément : Bonnes pratiques de rédaction et astuce **)

(**** Bonne pratique #0 : Revenez à la ligne régulièrement **)
(* Pas une longue ligne de tactiques, pas une seule tactique par ligne.
  L'idéal c'est de se fixer une largeur et de s'y tenir. *)

(**** Bonne pratique #1 : indenter vos preuves **)

(* Quand une tactique (ex [destruct],[split],[induction] ...) sépare votre but
 en plusieurs buts vous pouvez les distinguez par des tirets *)
(* C'est plus lisible, et un tiret permet de focus sur ce but, donc de voir son 
  contexte *)

Variable P : Prop.
Variable Q : Prop.
Variable R : Prop.

Goal P /\ Q -> Q /\ P.
Proof.
  intro H.
  destruct H.
  split.
   - exact H0.
   - exact H.
Qed.

(* Parfois vous séparez votre but plusieurs fois, 
   il faut donc des tirets différents.
   [-][+][*], mais aussi [--][++][**] etc. sont acceptés *)

Goal P /\ Q -> P /\ Q  /\ P /\ Q  /\ P /\ Q  /\ P /\ Q.
Proof.
  intro H.
  destruct H.
  split.
  - exact H.
  - split.
    + exact H0.
    + split.
      * exact H.
      * split.
        -- exact H0.
        -- split.
          --- exact H.
          --- split.
            ---- exact H0.
            ---- split.
              { exact H. }
              { exact H0. }
Qed.

(* Comme vous pouvez le voir, on peut aussi utiliser [{}] pour focus sur un but.
   Et on peut en imbriquer plusieurs ensemble, donc ça peut être plus élégant
   dans des preuves où on divise beaucoup, comme ci-dessus. *)

(**** Bonne pratique #2 : Nommez vos variables **)
Goal P /\ Q -> Q /\ P.
Proof.
  intro H.
  destruct H.
  split.
   - exact H0.
   - exact H.
Qed.

(* Dans cet exemple, [destruct H] crée deux variables, nommées automatiquement
H et H0. les variables suivantes seront nommés H1, H2 etc.
Problème, ce n'est pas robuste : supposons que vous avez écrit une looongue 
preuve, et que vous modifier son début, ou l'énoncé, ou un lemme qu'elle utilise.
Ça va décaler toute la notation automatique. Pour éviter ça : *)

Goal P /\ Q -> Q /\ P.
Proof.
  intro H.
  destruct H as [H H0].
  split.
   - exact H0.
   - exact H.
Qed.

(* Là c'est robuste, même si j'ai repris la même convention que Coq. Mais le 
  mieux c'est de donner des noms plus évocateurs*)

Goal P /\ Q -> Q /\ P.
Proof.
  intro H.
  destruct H as [HP HQ].
  split.
   - exact HQ.
   - exact HP.
Qed.

(* Autre exemple avec disjonction de but : *)

Goal P \/ Q -> Q \/ P.
Proof.
  intro H.
  destruct H as [H|H].
   - right. exact H.
   - left. exact H.
Qed.

(**** Astuce #0 : [Search] et [Print] **)

(* Dans une preuve vous avez besoin d'un lemme, que vous êtes à peu près sûr
  d'avoir prouvé plus tôt (ou qu'il existe dans la bibliothèque de Coq) mais 
  vous avez la flemme de remonter pour retrouver son nom. [Search] est votre ami *)
(*Avec [Search] je peux retrouver tous les énoncés qui contiennent un certain
  pattern .*)

Search (_ /\ _).
Search (_ \/ _ -> _ \/ _).

(* Pour revoir la définition ou le type d'un lemme/fonction dont vous connaissez
   le nom, c'est plutôt [Print]. *)

Print and_comm.

